var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Cache from "@tleef/expo-image-cache";
import React from "react";
import { Image, StyleSheet, View } from "react-native";
import Animated, { Easing } from "react-native-reanimated";
const cache = new Cache();
const { Value, timing } = Animated;
export default class ProgressiveImage extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            imageUri: undefined,
        };
        this.opacity = new Value(0);
        this.animFinished = false;
        this.anim = timing(this.opacity, {
            duration: props.animDuration,
            toValue: 1,
            easing: Easing.inOut(Easing.ease),
        });
        this.onImageLoad = this.onImageLoad.bind(this);
        this.onAnimFinished = this.onAnimFinished.bind(this);
    }
    componentDidMount() {
        this.resolveImageUri();
    }
    componentDidUpdate(prevProps) {
        let prevImageUri;
        if (prevProps.imageUri) {
            prevImageUri = prevProps.imageUri;
        }
        else if (prevProps.imageSource) {
            prevImageUri = prevProps.imageSource.uri;
        }
        let curImageUri;
        if (this.props.imageUri) {
            curImageUri = this.props.imageUri;
        }
        else if (this.props.imageSource) {
            curImageUri = this.props.imageSource.uri;
        }
        if (curImageUri !== prevImageUri) {
            this.reset();
            this.resolveImageUri();
        }
    }
    reset() {
        this.opacity.setValue(0);
        this.animFinished = false;
        this.setState({
            imageUri: undefined,
        });
    }
    resolveImageUri() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.props.noCache) {
                return;
            }
            let imageUri;
            if (this.props.imageUri) {
                imageUri = this.props.imageUri;
            }
            else if (this.props.imageSource) {
                imageUri = this.props.imageSource.uri;
            }
            if (!imageUri || !imageUri.startsWith("http")) {
                this.setState({
                    imageUri,
                });
                return;
            }
            try {
                const localImageUri = yield cache.getLocalUriAsync(imageUri);
                this.setState({
                    imageUri: localImageUri,
                });
            }
            catch (e) {
                console.log(`Error while caching image, using remote URI: ${imageUri}`);
                console.log(e);
                this.setState({
                    imageUri,
                });
            }
        });
    }
    render() {
        let { style } = this.props;
        return (<View style={[styles.container, style]}>
        {this.renderPreview()}
        {this.renderImage()}
      </View>);
    }
    renderPreview() {
        let { noPreview, previewBlurRadius } = this.props;
        let previewUri = this.props.previewUri;
        let previewSource = previewUri
            ? { uri: previewUri }
            : this.props.previewSource;
        if (noPreview || !previewSource) {
            return null;
        }
        return (<Image style={styles.image} blurRadius={previewBlurRadius} source={previewSource}/>);
    }
    renderImage() {
        let { noCache, noAnimation } = this.props;
        let imageSource;
        let imageUri;
        if (!noCache && (imageUri = this.state.imageUri)) {
            imageSource = { uri: imageUri };
        }
        else if (noCache && (imageUri = this.props.imageUri)) {
            imageSource = { uri: imageUri };
        }
        else if (noCache && this.props.imageSource) {
            imageSource = this.props.imageSource;
        }
        if (!imageSource) {
            return null;
        }
        if (noAnimation) {
            return <Image style={styles.image} source={imageSource}/>;
        }
        return (<Animated.Image style={[styles.image, { opacity: this.opacity }]} source={imageSource} onLoad={this.onImageLoad}/>);
    }
    onAnimFinished({ finished }) {
        this.animFinished = finished;
    }
    onImageLoad() {
        if (!this.animFinished) {
            this.anim.start(this.onAnimFinished);
        }
    }
}
ProgressiveImage.defaultProps = {
    noCache: false,
    noPreview: false,
    noAnimation: false,
    previewBlurRadius: 2,
    animDuration: 500,
};
const styles = StyleSheet.create({
    image: {
        position: "absolute",
        left: 0,
        right: 0,
        top: 0,
        bottom: 0,
    },
    container: {
        flex: 1,
        position: "relative",
    },
});
//# sourceMappingURL=ProgressiveImage.js.map